import * as React from "react";
import { Slot } from "@radix-ui/react-slot";
import { cva, type VariantProps } from "class-variance-authority";

import { cn } from "@/lib/utils";

const buttonVariants = cva(
  "inline-flex items-center justify-center gap-2 whitespace-nowrap rounded-lg text-sm font-medium ring-offset-background transition-all duration-300 ease-out focus-visible:outline-none focus-visible:ring-2 focus-visible:ring-ring focus-visible:ring-offset-2 disabled:pointer-events-none disabled:opacity-50",
  {
    variants: {
      variant: {
        default:
          "bg-primary text-white shadow-md hover:bg-primary/90 hover:shadow-lg hover:shadow-primary/20 hover:scale-[1.05]",
        destructive:
          "bg-destructive text-destructive-foreground shadow-sm hover:bg-destructive/90",
        outline:
          "border border-primary/50 bg-transparent hover:border-primary text-foreground dark:text-white shadow-sm hover:text-foreground dark:hover:text-white hover:scale-[1.05]",
        secondary:
          "bg-secondary text-secondary-foreground shadow-sm hover:bg-secondary/80",
        ghost: "hover:bg-accent hover:text-accent-foreground",
        link: "text-primary underline-offset-4 hover:underline",
        glow: "bg-primary text-white shadow-glow hover:bg-primary/90",
        gradient: "bg-gradient-to-r from-indigo-600 to-purple-600 text-white shadow-lg hover:from-indigo-700 hover:to-purple-700 hover:shadow-purple-500/50",
        action: "bg-primary hover:bg-primary/90 text-white font-medium shadow-md hover:shadow-lg hover:shadow-primary/20 transform hover:translate-y-[-2px]",
        outlineLight: "border border-gray-300 bg-transparent hover:bg-gray-100 text-gray-800 shadow-sm hover:text-gray-800 hover:border-gray-500",
        outlineDark: "border border-gray-700 bg-transparent hover:bg-gray-800 text-white shadow-sm hover:text-white hover:border-gray-500",
        // New variant for consistent "Kom i gang" style based on HeroHolo
        komIGang: "bg-primary text-white shadow-md hover:bg-primary/90 hover:shadow-lg hover:shadow-primary/20 border border-primary/50 hover:scale-[1.05]",
      },
      size: {
        default: "h-10 px-5 py-2.5",
        sm: "h-9 rounded-md px-4 py-2 text-xs",
        lg: "h-12 rounded-md px-7 py-3 text-base",
        xl: "h-14 rounded-md px-10 py-4 text-xl",
        icon: "h-10 w-10",
        pill: "rounded-full py-3 px-8 text-lg",
      },
      hasIcon: {
        true: "group",
        false: "",
      },
      iconPosition: {
        left: "",
        right: "",
      },
    },
    compoundVariants: [
      {
        hasIcon: true,
        iconPosition: "right",
        className: "[&>svg]:ml-2 [&>svg]:group-hover:translate-x-1 transition-transform",
      },
      {
        hasIcon: true,
        iconPosition: "left",
        className: "[&>svg]:mr-2 [&>svg]:group-hover:-translate-x-1 transition-transform", 
      },
    ],
    defaultVariants: {
      variant: "default",
      size: "default",
      hasIcon: false,
      iconPosition: "right",
    },
  }
);

export interface ButtonProps
  extends React.ButtonHTMLAttributes<HTMLButtonElement>,
    VariantProps<typeof buttonVariants> {
  asChild?: boolean;
  leftIcon?: React.ReactNode;
  rightIcon?: React.ReactNode;
}

const Button = React.forwardRef<HTMLButtonElement, ButtonProps>(
  ({ className, variant, size, hasIcon, iconPosition, asChild = false, leftIcon, rightIcon, children, ...props }, ref) => {
    const Comp = asChild ? Slot : "button";
    
    // Determine if we have icons and set the hasIcon prop accordingly
    const hasLeftIcon = !!leftIcon;
    const hasRightIcon = !!rightIcon;
    const computedHasIcon = hasIcon || hasLeftIcon || hasRightIcon;
    
    // Determine icon position - if both are present, respect the iconPosition prop
    const computedIconPosition = iconPosition || (hasLeftIcon && !hasRightIcon ? "left" : "right");
    
    // When asChild is true, we only pass children without icons to avoid the SlotClone error
    if (asChild) {
      return (
        <Comp
          className={cn(
            buttonVariants({ 
              variant, 
              size, 
              className 
            })
          )}
          ref={ref}
          {...props}
        >
          {children}
        </Comp>
      );
    }
    
    // When not asChild, we can safely include icons
    return (
      <Comp
        className={cn(
          buttonVariants({ 
            variant, 
            size, 
            hasIcon: computedHasIcon, 
            iconPosition: computedIconPosition, 
            className 
          })
        )}
        ref={ref}
        {...props}
      >
        {hasLeftIcon && leftIcon}
        {children}
        {hasRightIcon && rightIcon}
      </Comp>
    );
  }
);
Button.displayName = "Button";

export { Button, buttonVariants };
